@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
   
@@ -240,6 +240,16 @@ enum grad_avp_eval_type {
 				to evaluate it */
 };
 
+typedef struct grad_string {
+	size_t size;
+	char *data;
+} grad_string_t;
+
+typedef union grad_datum {
+	grad_uint32_t   ival;       /* integer value */
+	grad_string_t   sval;       /* string value */
+} grad_datum_t;
+
 /* An attribute/value pair */
 typedef struct grad_value_pair {
         struct grad_value_pair  *next;      /* Link to next A/V pair in list */
@@ -249,19 +259,11 @@ typedef struct grad_value_pair {
         enum grad_avp_eval_type eval_type;  /* Evaluation flag */
         int                     prop;       /* Properties */ 
         enum grad_operator operator;        /* Comparison operator */
-        union {
-                grad_uint32_t   ival;       /* integer value */
-                struct {
-                        int     s_length;   /* length of s_value w/o
-                                             * trailing 0
-                                             */
-                        char    *s_value;   /* string value */
-                } string;
-        } v;
-        
-#define avp_lvalue v.ival
-#define avp_strvalue v.string.s_value
-#define avp_strlength v.string.s_length
+	grad_datum_t            datum;      /* Actual data */
+
+#define avp_lvalue datum.ival
+#define avp_strvalue datum.sval.data
+#define avp_strlength datum.sval.size
 
 } grad_avp_t;
 
@@ -307,15 +309,15 @@ struct grad_matching_rule {
 
 /* External variables */
 
-extern char *radius_dir;
-extern char *radlog_dir;
-extern char *radacct_dir;
-extern char *radutmp_path;
-extern char *radwtmp_path;
-extern char *radstat_path;
-extern char *radmsgid_path;
-extern char *radpid_dir;
-extern char *bug_report_address;
+extern char *grad_config_dir;
+extern char *grad_log_dir;
+extern char *grad_acct_dir;
+extern char *grad_utmp_file;
+extern char *grad_wtmp_file;
+extern char *grad_stat_file;
+extern char *grad_msgid_file;
+extern char *grad_pid_dir;
+extern char *grad_bug_report_address;
 
 /* Parser */
 extern grad_locus_t grad_parser_source_locus;
@@ -392,7 +394,7 @@ int grad_avp_cmp(grad_avp_t *a, grad_avp_t *b);
 int grad_avl_cmp(grad_avp_t *a, grad_avp_t *b, int prop);
 int grad_avp_null_string_p(grad_avp_t *pair);
 	
-extern int resolve_hostnames;
+extern int grad_resolve_hostnames;
 
 char *grad_ip_gethostname (grad_uint32_t, char *buf, size_t size);
 grad_uint32_t grad_ip_gethostaddr (const char *);
@@ -452,7 +454,8 @@ int grad_format_string_visual(char *buf, int runlen, char *str, int len);
 char *grad_op_to_str(enum grad_operator op);
 enum grad_operator grad_str_to_op(char *str);
 int grad_xlat_keyword(grad_keyword_t *kw, const char *str, int def);
-void grad_obstack_grow_backslash_num(struct obstack *stk, char *text,
+void grad_obstack_grow_backslash_num(struct obstack *stk,
+				     char *text, char **pend,
 				     int len, int base);
 void grad_obstack_grow_backslash(struct obstack *stk, char *text,
 				 char **endp);
