@@ -13,9 +13,10 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
   
-   You should have received a copy of the GNU General Public License
-   along with GNU Radius; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+   You should have received a copy of the GNU General Public
+   License along with GNU Radius; if not, write to the Free
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301 USA. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -68,7 +69,7 @@ grad_pdu_init(struct radius_pdu *pdu)
           authenticator -- Request authenticator.
    Output:
           *ptr   -- Radius reply.
-   Return value: length of the data on *ptr. */   
+   Return value: length of the data on *ptr. */
 static size_t
 grad_pdu_finish(void **ptr, struct radius_pdu *pdu,
 		int code, int id, u_char *authenticator, u_char *secret)
