@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
  
@@ -51,9 +51,9 @@ static int write_wtmp(struct radutmp *ut);
 static int write_nas_restart(int status, grad_uint32_t addr);
 static int check_ts(struct radutmp *ut);
 
-int rad_acct_system(grad_request_t *radreq, int dowtmp);
-int rad_acct_db(grad_request_t *radreq, int authtype);
-int rad_acct_ext(grad_request_t *radreq);
+int rad_acct_system(radiusd_request_t *radreq, int dowtmp);
+int rad_acct_db(radiusd_request_t *radreq, int authtype);
+int rad_acct_ext(radiusd_request_t *radreq);
 
 
 /* Zap a user, or all users on a NAS, from the radutmp file. */
@@ -67,9 +67,9 @@ radzap(grad_uint32_t nasaddr, int port, char *user, time_t t)
         if (t == 0) time(&t);
         netaddr = htonl(nasaddr);
 
-        if (file = rut_setent(radutmp_path, 0)) {
+        if (file = grad_ut_setent(grad_utmp_file, 0)) {
                 /* Find the entry for this NAS / portno combination. */
-                while (up = rut_getent(file)) {
+                while (up = grad_ut_getent(file)) {
                         if (((nasaddr != 0 && netaddr != up->nas_address) ||
                              (port >= 0   && port    != up->nas_port) ||
                              (user != NULL && strcmp(up->login, user) != 0) ||
@@ -78,11 +78,11 @@ radzap(grad_uint32_t nasaddr, int port, char *user, time_t t)
                         /* Zap the entry */
                         up->type = P_IDLE;
                         up->time = t;
-                        rut_putent(file, up);
+                        grad_ut_putent(file, up);
                         /* Add a logout entry to the wtmp file. */
                         write_wtmp(up);
                 }
-                rut_endent(file);
+                grad_ut_endent(file);
         }
 
         return 0;
@@ -101,7 +101,7 @@ store_session_id(char *buffer, int len, char *id, int idlen)
 int
 write_wtmp(struct radutmp *ut)
 {
-        return radwtmp_putent(radwtmp_path, ut);
+        return grad_radwtmp_putent(grad_wtmp_file, ut);
 }
 
 void
@@ -175,7 +175,7 @@ check_attribute(grad_avp_t *check_pairs, int pair_attr,
 
 /*  Store logins in the RADIUS utmp file. */
 int
-rad_acct_system(grad_request_t *radreq, int dowtmp)
+rad_acct_system(radiusd_request_t *radreq, int dowtmp)
 {
         struct radutmp  ut;
         grad_avp_t *vp;
@@ -185,16 +185,16 @@ rad_acct_system(grad_request_t *radreq, int dowtmp)
         time_t t;
         int ret = 0, rc;
         int port_seen = 0;
-        char buf[MAX_LONGNAME];
+        char buf[GRAD_MAX_LONGNAME];
 
 	/* Do not do anything if system accounting is not requested */
 	if (!acct_system)
 		return 0;
 	
         /* A packet should have Acct-Status-Type attribute */
-        if ((vp = grad_avl_find(radreq->request, DA_ACCT_STATUS_TYPE))
+        if ((vp = grad_avl_find(radreq->request->avlist, DA_ACCT_STATUS_TYPE))
 	        == NULL) {
-                grad_log_req(L_ERR, radreq,
+                grad_log_req(L_ERR, radreq->request,
                              _("no Acct-Status-Type attribute"));
                 return -1;
         }
@@ -214,7 +214,7 @@ rad_acct_system(grad_request_t *radreq, int dowtmp)
         }
         
         /* Fill in radutmp structure */
-        for (vp = radreq->request; vp; vp = vp->next) {
+        for (vp = radreq->request->avlist; vp; vp = vp->next) {
                 switch (vp->attribute) {
                 case DA_USER_NAME:
                         backslashify(ut.login, vp->avp_strvalue, RUT_NAMESIZE);
@@ -276,7 +276,7 @@ rad_acct_system(grad_request_t *radreq, int dowtmp)
         /* If we didn't find out the NAS address, use the originator's
            IP address. */
         if (nas_address == 0) {
-                nas_address = radreq->ipaddr;
+                nas_address = radreq->request->ipaddr;
                 ut.nas_address = htonl(nas_address);
         }
 
@@ -316,7 +316,7 @@ rad_acct_system(grad_request_t *radreq, int dowtmp)
             status != DV_ACCT_STATUS_TYPE_STOP &&
             status != DV_ACCT_STATUS_TYPE_ALIVE) {
                 grad_log_req(L_NOTICE, 
-                             radreq, _("unknown packet type (%d)"),
+                             radreq->request, _("unknown packet type (%d)"),
                              status);
                 return 0;
         } else if (status == DV_ACCT_STATUS_TYPE_START ||
@@ -343,14 +343,14 @@ rad_acct_system(grad_request_t *radreq, int dowtmp)
         if (!port_seen)
                 return 0;
 
-        if (!ACCT_TYPE(radreq->request, DV_ACCT_TYPE_SYSTEM)) {
+        if (!ACCT_TYPE(radreq->request->avlist, DV_ACCT_TYPE_SYSTEM)) {
                 debug(1,("Acct type system disabled for %s", ut.login));
                 return 0;
         }
         
         /* Update radutmp file. */
-        rc = radutmp_putent(radutmp_path, &ut, status);
-        debug(1, ("radutmp_putent=%d for %s/%s",
+        rc = grad_utmp_putent(grad_utmp_file, &ut, status);
+        debug(1, ("grad_utmp_putent=%d for %s/%s",
                   rc,
                   ut.login,
                   ut.session_id));
@@ -370,8 +370,9 @@ rad_acct_system(grad_request_t *radreq, int dowtmp)
                 stat_update(&ut, status);
         } else {
                 ret = -1;
-                stat_inc(acct, radreq->ipaddr, num_norecords);
-                grad_log_req(L_NOTICE, radreq, _("NOT writing wtmp record"));
+                stat_inc(acct, radreq->request->ipaddr, num_norecords);
+                grad_log_req(L_NOTICE, radreq->request,
+			     _("NOT writing wtmp record"));
         }
         return ret;
 }
@@ -383,15 +384,15 @@ disable_system_acct()
 	struct radutmp *up;
 	int written = 0;
 		
-        if ((file = rut_setent(radutmp_path, 0)) == NULL)
+        if ((file = grad_ut_setent(grad_utmp_file, 0)) == NULL)
                 return;
 
-        while (up = rut_getent(file)) {
+        while (up = grad_ut_getent(file)) {
                 switch (up->type) {
 		case P_LOGIN:
                         up->type = P_IDLE;
                         time(&up->time);
-                        rut_putent(file, up);
+                        grad_ut_putent(file, up);
                         /* Add a logout entry to the wtmp file. */
                         write_wtmp(up);
 			break;
@@ -399,7 +400,7 @@ disable_system_acct()
 		case P_ACCT_ENABLED:
 			time(&up->time);
 			up->type = P_ACCT_DISABLED;
-			rut_putent(file, up);
+			grad_ut_putent(file, up);
 			write_wtmp(up);
 			written = 1;
 			break;
@@ -416,9 +417,9 @@ disable_system_acct()
 		memset(&ut, 0, sizeof ut);
 		time(&ut.time);
 		ut.type = P_ACCT_DISABLED;
-		rut_putent(file, &ut);
+		grad_ut_putent(file, &ut);
 	}
-	rut_endent(file);
+	grad_ut_endent(file);
 	grad_log(L_INFO, _("System accounting is disabled"));
 }
 
@@ -428,19 +429,19 @@ enable_system_acct()
         radut_file_t file;
 	struct radutmp *up, ut;
 		
-        if ((file = rut_setent(radutmp_path, 0)) == NULL)
+        if ((file = grad_ut_setent(grad_utmp_file, 0)) == NULL)
                 return;
 
-        while (up = rut_getent(file)) {
+        while (up = grad_ut_getent(file)) {
                 if (up->type == P_ACCT_DISABLED) {
                         up->type = P_ACCT_ENABLED;
                         time(&up->time);
-                        rut_putent(file, up);
+                        grad_ut_putent(file, up);
                         /* Add an entry to the wtmp file. */
                         write_wtmp(up);
                 }
 	}
-	rut_endent(file);
+	grad_ut_endent(file);
 }
 
 void
@@ -497,23 +498,23 @@ check_acct_dir()
 {
         struct stat st;
 
-	if (stat(radacct_dir, &st) == 0) {
+	if (stat(grad_acct_dir, &st) == 0) {
 		if (S_ISDIR(st.st_mode)) {
-			if (access(radacct_dir, W_OK)) {
+			if (access(grad_acct_dir, W_OK)) {
 				grad_log(L_ERR,
 					 _("%s: directory not writable"),
-					 radacct_dir);
+					 grad_acct_dir);
 				return 1;
 			}
 			return 0;
 		} else {
 			grad_log(L_ERR, _("%s: not a directory"),
-				 radacct_dir);
+				 grad_acct_dir);
 			return 1;
 		}
 	}
 
-	if (mkdir_path(radacct_dir, 0755)) 
+	if (mkdir_path(grad_acct_dir, 0755)) 
                 return 1;
         return 0;
 }
@@ -540,9 +541,9 @@ acct_init()
 }
 
 static char *
-make_legacy_detail_filename(grad_request_t *radreq, char *default_filename)
+make_legacy_detail_filename(radiusd_request_t *radreq, char *default_filename)
 {
-	char nasname[MAX_LONGNAME];
+	char nasname[GRAD_MAX_LONGNAME];
 	grad_nas_t *cl;
 	grad_uint32_t ip;
 	grad_avp_t *pair;
@@ -553,8 +554,8 @@ make_legacy_detail_filename(grad_request_t *radreq, char *default_filename)
            Only if that fails we resort to a name lookup. */
 	
         cl = NULL;
-        ip = radreq->ipaddr;
-        if ((pair = grad_avl_find(radreq->request, DA_NAS_IP_ADDRESS)) != NULL)
+        ip = radreq->request->ipaddr;
+        if ((pair = grad_avl_find(radreq->request->avlist, DA_NAS_IP_ADDRESS)) != NULL)
                 ip = pair->avp_lvalue;
 
         if (radreq->realm) {
@@ -579,30 +580,29 @@ make_legacy_detail_filename(grad_request_t *radreq, char *default_filename)
 }
 
 static char *
-make_detail_filename(grad_request_t *req, char *template,
+make_detail_filename(radiusd_request_t *req, char *template,
 		     char *default_filename)
 {
 	if (!template) {
 		return make_legacy_detail_filename(req, default_filename);
 	} else if (template[0] == '=') {
-		Datatype type;
-		Datum datum;
-
+		grad_value_t val;
 		/*FIXME: Should be compiled!*/
-		if (rewrite_interpret(template+1, req, &type, &datum)) 
+		if (rewrite_interpret(template+1, req->request, &val)) 
 			return NULL;
-		if (type != String) {
+		if (val.type != String) {
 			grad_log(L_ERR, "%s: %s",
 			         template+1, _("wrong return type"));
+			/* Nothing to free in val */
 			return NULL;
 		}
-		return datum.sval;
+		return val.datum.sval.data;
 	} else {
 		struct obstack stk;
 		char *ptr;
 		
 		obstack_init(&stk);
-		ptr = radius_xlate(&stk, template, req, NULL);
+		ptr = radius_xlate(&stk, template, req->request, NULL);
 		if (ptr) 
 			ptr = grad_estrdup(ptr);
 		obstack_free(&stk, NULL);
@@ -625,7 +625,7 @@ make_path_to_file(char *filename, int perms)
 }
 
 int
-write_detail(grad_request_t *radreq, int authtype, int rtype)
+write_detail(radiusd_request_t *radreq, int authtype, int rtype)
 {
         FILE *outfd;
         char *save;
@@ -661,7 +661,7 @@ write_detail(grad_request_t *radreq, int authtype, int rtype)
 		return 1;
 
 	/* Change to the accounting directory */
-	if (chdir(radacct_dir)) {
+	if (chdir(grad_acct_dir)) {
 		grad_free(filename);
 		return 1;
 	}
@@ -683,12 +683,12 @@ write_detail(grad_request_t *radreq, int authtype, int rtype)
                         /* user wants a full (non-stripped) name to appear
                            in detail */
                         
-                        pair = grad_avl_find(radreq->request,
+                        pair = grad_avl_find(radreq->request->avlist,
 					     DA_ORIG_USER_NAME);
                         if (pair) 
                                 pair->name = "User-Name";
                         else
-                                pair = grad_avl_find(radreq->request,
+                                pair = grad_avl_find(radreq->request->avlist,
 						     DA_USER_NAME);
                         if (pair) {
                                 fprintf(outfd, "\t%s\n", 
@@ -698,8 +698,8 @@ write_detail(grad_request_t *radreq, int authtype, int rtype)
                 }
 
                 /* Write each attribute/value to the log file */
-                for (pair = radreq->request; pair; pair = pair->next) {
-			if (pair->prop & AP_INTERNAL)
+                for (pair = radreq->request->avlist; pair; pair = pair->next) {
+			if (pair->prop & GRAD_AP_INTERNAL)
 				continue;
                         switch (pair->attribute) {
                         case DA_USER_PASSWORD:
@@ -743,45 +743,49 @@ write_detail(grad_request_t *radreq, int authtype, int rtype)
 }
 
 int
-rad_acct_db(grad_request_t *radreq, int authtype)
+rad_acct_db(radiusd_request_t *radreq, int authtype)
 {
         int rc = 0;
 
-        if (acct_detail && ACCT_TYPE(radreq->request, DV_ACCT_TYPE_DETAIL))
+        if (acct_detail
+	    && ACCT_TYPE(radreq->request->avlist, DV_ACCT_TYPE_DETAIL))
                 rc = write_detail(radreq, authtype, R_ACCT);
-        if (ACCT_TYPE(radreq->request, DV_ACCT_TYPE_SQL))
+        if (ACCT_TYPE(radreq->request->avlist, DV_ACCT_TYPE_SQL))
                 radiusd_sql_acct(radreq);
         return rc;
 }
 
 int
-rad_acct_ext(grad_request_t *radreq)
+rad_acct_ext(radiusd_request_t *radreq)
 {
         grad_avp_t *p;
-
+	int rc = 0;
+	
 #ifdef USE_SERVER_GUILE
-        for (p = grad_avl_find(radreq->request, DA_SCHEME_ACCT_PROCEDURE);
+        for (p = grad_avl_find(radreq->request->avlist, DA_SCHEME_ACCT_PROCEDURE);
 	     p;
 	     p = grad_avl_find(p->next, DA_SCHEME_ACCT_PROCEDURE))
                 scheme_acct(p->avp_strvalue, radreq);
 #endif
-        for (p = grad_avl_find(radreq->request, DA_ACCT_EXT_PROGRAM);
+        for (p = grad_avl_find(radreq->request->avlist, DA_ACCT_EXT_PROGRAM);
 	     p;
 	     p = grad_avl_find(p->next, DA_ACCT_EXT_PROGRAM)) {
+		radius_eval_avp(radreq, p, NULL, 1);
     		switch (p->avp_strvalue[0]) {
 		case '/':
-                	radius_exec_program(p->avp_strvalue, radreq, NULL, 0);
+                	rc = radius_exec_program(p->avp_strvalue, radreq, NULL,
+						 1);
 			break;
 		case '|':
                 	filter_acct(p->avp_strvalue+1, radreq);
                 }
         }
-        return 0;
+        return rc;
 }
 
 /* run accounting modules */
 int
-rad_accounting(grad_request_t *radreq, int activefd, int verified)
+rad_accounting(radiusd_request_t *radreq, int activefd, int verified)
 {
 	log_open(L_ACCT);
 
@@ -789,11 +793,11 @@ rad_accounting(grad_request_t *radreq, int activefd, int verified)
 
 #if defined(RT_ASCEND_EVENT_REQUEST) && defined(RT_ASCEND_EVENT_RESPONSE)
         /* Special handling for Ascend-Event-Request */
-        if (radreq->code == RT_ASCEND_EVENT_REQUEST) {
+        if (radreq->request->code == RT_ASCEND_EVENT_REQUEST) {
                 write_detail(radreq, verified, R_ACCT);
                 radius_send_reply(RT_ASCEND_EVENT_RESPONSE,
                                   radreq, NULL, NULL, activefd);
-                stat_inc(acct, radreq->ipaddr, num_resp);
+                stat_inc(acct, radreq->request->ipaddr, num_resp);
                 return 0;
         }
 #endif
@@ -804,7 +808,7 @@ rad_accounting(grad_request_t *radreq, int activefd, int verified)
                 /* Now send back an ACK to the NAS. */
                 radius_send_reply(RT_ACCOUNTING_RESPONSE,
                                radreq, NULL, NULL, activefd);
-                stat_inc(acct, radreq->ipaddr, num_resp);
+                stat_inc(acct, radreq->request->ipaddr, num_resp);
                 return 0;
         }
 
@@ -830,16 +834,16 @@ write_nas_restart(int status, grad_uint32_t addr)
 /* Multiple login checking */
 
 int
-radutmp_mlc_collect_user(char *name, grad_request_t *request,
+radutmp_mlc_collect_user(char *name, radiusd_request_t *request,
 			 grad_list_t **sess_list)
 {
         radut_file_t file;
 	struct radutmp *up;
 		
-        if ((file = rut_setent(radutmp_path, 0)) == NULL)
+        if ((file = grad_ut_setent(grad_utmp_file, 0)) == NULL)
                 return 1;
 
-        while (up = rut_getent(file)) {
+        while (up = grad_ut_getent(file)) {
                 if (strncmp(name, up->login, RUT_NAMESIZE) == 0 &&
                     up->type == P_LOGIN) {
 			struct radutmp *tmp;
@@ -851,20 +855,20 @@ radutmp_mlc_collect_user(char *name, grad_request_t *request,
 			grad_list_append(*sess_list, tmp);
                 }
 	}
-	rut_endent(file);
+	grad_ut_endent(file);
 	return 0;
 }
 
 int
-radutmp_mlc_collect_realm(grad_request_t *request, grad_list_t **sess_list)
+radutmp_mlc_collect_realm(radiusd_request_t *request, grad_list_t **sess_list)
 {
         radut_file_t file;
 	struct radutmp *up;
 		
-        if ((file = rut_setent(radutmp_path, 0)) == NULL)
+        if ((file = grad_ut_setent(grad_utmp_file, 0)) == NULL)
                 return 1;
 
-        while (up = rut_getent(file)) {
+        while (up = grad_ut_getent(file)) {
                 if (up->type == P_LOGIN
 		    && grad_realm_verify_ip(request->realm, up->realm_address)) {
 			struct radutmp *tmp;
@@ -876,7 +880,7 @@ radutmp_mlc_collect_realm(grad_request_t *request, grad_list_t **sess_list)
 			grad_list_append(*sess_list, tmp);
                 }
 	}
-	rut_endent(file);
+	grad_ut_endent(file);
 	return 0;
 }
 
@@ -885,7 +889,7 @@ radutmp_mlc_close(struct radutmp *up)
 {
 	up->type = P_IDLE;
 	up->time = time(NULL);
-	radutmp_putent(radutmp_path, up, DV_ACCT_STATUS_TYPE_STOP);
+	grad_utmp_putent(grad_utmp_file, up, DV_ACCT_STATUS_TYPE_STOP);
 }
 
 int
