@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius
-   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
   
@@ -93,11 +93,11 @@ create_instance(struct check_instance *cptr, grad_nas_t *nas, struct radutmp *up
                          nas->shortname);
                 return NULL;
         }
-        cptr->name = up->orig_login;
+        cptr->name = up->orig_login[0] ? up->orig_login : up->login;
         cptr->port = up->nas_port;
         cptr->sid  = up->session_id;
-        cptr->framed_ip = up->framed_address;
-	cptr->nas_ip = up->nas_address;
+        cptr->framed_ip = ntohl(up->framed_address);
+	cptr->nas_ip = ntohl(up->nas_address);
 	cptr->result = -1;
         cptr->timeout = 0;
         cptr->nasname = nas->shortname ? nas->shortname : nas->longname;
@@ -120,9 +120,9 @@ free_instance(struct check_instance *cptr)
 int
 compare(struct check_instance *checkp, char *str)
 {
-	grad_uint32_t result;
+	grad_value_t val;
 
-	if (rewrite_invoke(Integer, &result,
+	if (rewrite_invoke(Integer, &val,
 			   checkp->func, NULL,
 			   "ssis",
 			   str,
@@ -130,7 +130,7 @@ compare(struct check_instance *checkp, char *str)
 			   checkp->port,
 			   checkp->sid))
 		return -1;
-	return result;
+	return val.datum.ival;
 }
 
 
