@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
 
@@ -368,7 +368,7 @@ match_user(User_symbol *sym, radiusd_request_t *req,
                         debug(1, ("submatch: %s", p->avp_strvalue));
 
                         found = match_user(grad_sym_lookup(user_tab,
-						      p->avp_strvalue),
+							   p->avp_strvalue),
                                            req, check_pairs, reply_pairs);
                 }                       
 
@@ -607,8 +607,7 @@ avl_eval_rewrite (radiusd_request_t *req, grad_matching_rule_t *rule,
 {
 	for (; p; p = p->next) {
 		if (p->attribute == DA_REWRITE_FUNCTION
-		    && (rewrite_eval(p->avp_strvalue, req->request,
-				     NULL, NULL))) {
+		    && rewrite_eval(p->avp_strvalue, req->request, NULL)) {
 			grad_log_loc(L_ERR, &rule->loc, "%s(): %s",
 				     p->avp_strvalue,
 				     _("not defined"));
@@ -623,6 +622,47 @@ rule_eval_rewrite (radiusd_request_t *req, grad_matching_rule_t *rule)
 	avl_eval_rewrite (req, rule, rule->lhs);
 }
 
+int
+exec_program_wait (radiusd_request_t *request, grad_avp_t *rhs,
+		   grad_avp_t **reply, grad_avp_t **pfailed)
+{
+	int rc = 0;
+	grad_avp_t *p;
+	
+	for (p = rhs; p; p = p->next) {
+		if (p->attribute == DA_EXEC_PROGRAM_WAIT) {
+			radius_eval_avp(request, p, reply ? *reply : NULL, 1);
+			switch (p->avp_strvalue[0]) {
+			case '/':
+				rc = radius_exec_program(p->avp_strvalue,
+							 request,
+							 reply,
+							 1);
+				break;
+				
+			case '|':
+				rc = filter_auth(p->avp_strvalue+1,
+						 request,
+						 reply);
+				break;
+
+			default:
+				grad_log_req(L_ERR, request->request,
+					     _("cannot execute `%s'"),
+					     p->avp_strvalue);
+				rc = 1;
+			}
+
+			if (rc) {
+				if (pfailed)
+					*pfailed = p;
+				break;
+			}
+		}
+	}
+	return rc;
+}
+
 /* ***************************************************************************
  * raddb/hints
  */
@@ -630,9 +670,10 @@ rule_eval_rewrite (radiusd_request_t *req, grad_matching_rule_t *rule)
 /* Provide a support for backward-compatible attributes Replace-User-Name
    and Rewrite-Function */
 static void
-hints_eval_compat(radiusd_request_t *req, grad_avp_t *name_pair, grad_matching_rule_t *rule)
+hints_eval_compat(radiusd_request_t *req, grad_avp_t *name_pair,
+		  grad_matching_rule_t *rule)
 {
-        grad_avp_t      *tmp;
+        grad_avp_t *tmp;
 	
 	/* Let's see if we need to further modify the username */
 	if ((tmp = grad_avl_find(rule->rhs, DA_REPLACE_USER_NAME))
@@ -657,13 +698,13 @@ hints_eval_compat(radiusd_request_t *req, grad_avp_t *name_pair, grad_matching_r
 int
 hints_setup(radiusd_request_t *req)
 {
-        grad_avp_t      *request_pairs = req->request->avlist;
-        char            newname[GRAD_STRING_LENGTH];
-        grad_avp_t      *name_pair;
-        grad_avp_t      *orig_name_pair;
-        grad_avp_t      *tmp;
-        grad_matching_rule_t   *rule;
-        int             matched = 0;
+        char newname[GRAD_STRING_LENGTH];
+        grad_avp_t *name_pair;
+        grad_avp_t *orig_name_pair;
+	grad_avp_t *tmp_name_pair;
+        grad_avp_t *tmp;
+        grad_matching_rule_t *rule;
+        int matched = 0;
 	grad_iterator_t *itr;
 	
         /* Add Proxy-Replied pair if necessary */
@@ -673,22 +714,23 @@ hints_setup(radiusd_request_t *req)
         case RT_ACCOUNTING_RESPONSE:
         case RT_ACCESS_CHALLENGE:
                 tmp = grad_avp_create_integer(DA_PROXY_REPLIED, 1);
-                grad_avl_merge(&request_pairs, &tmp);
+                grad_avl_merge(&req->request->avlist, &tmp);
                 grad_avp_free(tmp);
                 break;
 
         case RT_STATUS_SERVER:
                 return 0;
         }
-        
+       
         if (hints == NULL)
                 return 0;
 
-        /* 
-         *      Check for valid input, zero length names not permitted 
-         */
-        if ((name_pair = grad_avl_find(request_pairs, DA_USER_NAME)) == NULL) {
-                name_pair = grad_avp_create_string(DA_USER_NAME, "");
+        /* Get User-Name pair. If it is absent, provide a replacement
+	   value. */
+        if ((name_pair = grad_avl_find(req->request->avlist,
+				       DA_USER_NAME)) == NULL) {
+		tmp_name_pair = grad_avp_create_string(DA_USER_NAME, "");
+                name_pair = tmp_name_pair;
                 orig_name_pair = NULL;
         } else {
                 orig_name_pair = grad_avp_create_string(DA_ORIG_USER_NAME,
@@ -699,12 +741,12 @@ hints_setup(radiusd_request_t *req)
         
         /* if Framed-Protocol is present but Service-Type is missing, add
            Service-Type = Framed-User. */
-        if (grad_avl_find(request_pairs, DA_FRAMED_PROTOCOL) != NULL &&
-            grad_avl_find(request_pairs, DA_SERVICE_TYPE) == NULL) {
+        if (grad_avl_find(req->request->avlist, DA_FRAMED_PROTOCOL) != NULL &&
+            grad_avl_find(req->request->avlist, DA_SERVICE_TYPE) == NULL) {
                 tmp = grad_avp_create_integer(DA_SERVICE_TYPE,
 					      DV_SERVICE_TYPE_FRAMED_USER);
                 if (tmp) 
-                        grad_avl_merge(&request_pairs, &tmp);
+                        grad_avl_merge(&req->request->avlist, &tmp);
         }
 
 	itr = grad_iterator_create(hints);
@@ -736,22 +778,33 @@ hints_setup(radiusd_request_t *req)
 
 		/* Handle Rewrite-Function and Replace-User-Name */
 		hints_eval_compat(req, name_pair, rule);
+		/* Process eventual Exec-Program-Wait and Scheme-Procedure
+		   attributes */
+		exec_program_wait (req, rule->rhs, &add, NULL);
+#ifdef USE_SERVER_GUILE
+		scheme_eval_avl (req, rule->lhs, rule->rhs, &add, NULL);
+#endif
 		/* Evaluate hints */
 		radius_eval_avl(req, add); /*FIXME: return value?*/
-		
-                debug(1, ("new name is `%s'", name_pair->avp_strvalue));
-
-                /* fix-up the string length */
-                name_pair->avp_strlength = strlen(name_pair->avp_strvalue);
 
                 /* Add all attributes to the request list, except
 		   DA_STRIP_USER_NAME and DA_REPLACE_USER_NAME */
                 grad_avl_delete(&add, DA_STRIP_USER_NAME);
                 grad_avl_delete(&add, DA_REPLACE_USER_NAME);
                 grad_avl_delete(&add, DA_REWRITE_FUNCTION);
-                grad_avl_merge(&request_pairs, &add);
+                grad_avl_merge(&req->request->avlist, &add);
                 grad_avl_free(add);
                 
+		/* Re-read the name pair, since it might have been changed
+		   by exec_program_wait */
+		tmp = grad_avl_find(req->request->avlist, DA_USER_NAME);
+		if (tmp)
+			name_pair = tmp;
+                debug(1, ("new name is `%s'", name_pair->avp_strvalue));
+
+                /* fix-up the string length. FIXME: Do we still need it? */
+                name_pair->avp_strlength = strlen(name_pair->avp_strvalue);
+
                 /* Ok, let's see if we need to further check the
                    hint's rules */
                 if (((tmp = grad_avl_find(rule->rhs, DA_FALL_THROUGH))
@@ -767,22 +820,22 @@ hints_setup(radiusd_request_t *req)
 	
         if (matched) {
                 if (orig_name_pair)
-                        grad_avl_add_pair(&request_pairs, orig_name_pair);
-		/* A rewrite function might have installed the new
+                        grad_avl_add_pair(&req->request->avlist,
+					  orig_name_pair);
+		/* A rewrite function might have installed a new
 		   User-Name. Take care not to discard it. */
-                else if (!grad_avl_find(request_pairs, DA_USER_NAME))
-                        grad_avl_add_pair(&request_pairs, name_pair);
+                else if (!grad_avl_find(req->request->avlist, DA_USER_NAME))
+                        grad_avl_add_pair(&req->request->avlist,
+					  tmp_name_pair);
 		else
-			grad_avp_free(name_pair);
+			grad_avp_free(tmp_name_pair);
         } else {
                 if (orig_name_pair)
                         grad_avp_free(orig_name_pair);
                 else
-                        grad_avp_free(name_pair);
+                        grad_avp_free(tmp_name_pair);
         }
 
-        req->request->avlist = request_pairs;
-        
         return 0;
 }
 
@@ -1096,7 +1149,7 @@ read_deny_file()
         int denycnt;
         char *name;
         
-        name = grad_mkfilename(radius_dir, RADIUS_DENY);
+        name = grad_mkfilename(grad_config_dir, RADIUS_DENY);
         if (deny_tab)
                 grad_symtab_clear(deny_tab);
         else
@@ -1643,7 +1696,7 @@ reload_data(enum reload_what what, int *do_radck)
 
         case reload_users:
                 grad_symtab_clear(user_tab);
-                path = grad_mkfilename(radius_dir, RADIUS_USERS);
+                path = grad_mkfilename(grad_config_dir, RADIUS_USERS);
         
 #if USE_DBM
                 if (use_dbm && radius_mode != MODE_BUILDDBM) {
@@ -1686,14 +1739,14 @@ reload_data(enum reload_what what, int *do_radck)
                 
         case reload_huntgroups:
                 grad_list_destroy(&huntgroups, matching_rule_free, NULL);
-                path = grad_mkfilename(radius_dir, RADIUS_HUNTGROUPS);
+                path = grad_mkfilename(grad_config_dir, RADIUS_HUNTGROUPS);
                 huntgroups = file_read(GRAD_CF_HUNTGROUPS, path);
                 grad_free(path);
                 break;
                 
         case reload_hints:
                 grad_list_destroy(&hints, matching_rule_free, NULL);
-                path = grad_mkfilename(radius_dir, RADIUS_HINTS);
+                path = grad_mkfilename(grad_config_dir, RADIUS_HINTS);
                 hints = file_read(GRAD_CF_HINTS, path);
                 grad_free(path);
                 if (!use_dbm) 
@@ -1701,7 +1754,7 @@ reload_data(enum reload_what what, int *do_radck)
                 break;
                 
         case reload_clients:
-                path = grad_mkfilename(radius_dir, RADIUS_CLIENTS);
+                path = grad_mkfilename(grad_config_dir, RADIUS_CLIENTS);
                 if (read_clients_file(path) < 0)
                         rc = 1;
                 grad_free(path);
@@ -1709,19 +1762,19 @@ reload_data(enum reload_what what, int *do_radck)
 
         case reload_naslist:
                 /*FIXME*/
-		path = grad_mkfilename(radius_dir, RADIUS_NASTYPES);
+		path = grad_mkfilename(grad_config_dir, RADIUS_NASTYPES);
                 read_nastypes_file(path);
                 grad_free(path);
                 /*EMXIF*/
 
-                path = grad_mkfilename(radius_dir, RADIUS_NASLIST);
+                path = grad_mkfilename(grad_config_dir, RADIUS_NASLIST);
                 if (read_naslist_file(path) < 0)
                         rc = 1;
                 grad_free(path);
                 break;
 
         case reload_realms:
-                path = grad_mkfilename(radius_dir, RADIUS_REALMS);
+                path = grad_mkfilename(grad_config_dir, RADIUS_REALMS);
                 if (grad_read_realms(path, auth_port, acct_port,
 				     realm_set_secret) < 0)
                         rc = 1;
@@ -1817,7 +1870,7 @@ void
 dump_users_db()
 {
         FILE *fp;
-        char *name = grad_mkfilename(radlog_dir, RADIUS_DUMPDB_NAME);
+        char *name = grad_mkfilename(grad_log_dir, RADIUS_DUMPDB_NAME);
         
         fp = fopen(name, "w");
         if (!fp) {
