@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004,2005 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
   
@@ -27,8 +27,8 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <ctype.h>
-
 #include <radiusd.h>
+
 #ifdef USE_SERVER_GUILE
 # include <libguile.h>
 #endif
@@ -41,7 +41,7 @@ static void detach_sql_connection(int type);
 static void sql_conn_destroy(struct sql_connection **conn);
 static void sql_cache_destroy(struct sql_connection *conn);
 
-static char *getline();
+static char *get_conf_line();
 static int get_boolean(char *str, int *retval);
 static char * sql_digest(SQL_cfg *cfg);
 static int sql_digest_comp(char *d1, char *d2);
@@ -144,7 +144,7 @@ chop(char *str)
 }
 
 char *
-getline()
+get_conf_line()
 {
         char buf[256];
         char *ptr;
@@ -310,7 +310,7 @@ radiusd_sql_config()
         new_cfg.active[SQL_ACCT] = 0;
         new_cfg.active[SQL_AUTH] = 0;
 
-        sqlfile = grad_mkfilename(radius_dir, "sqlserver");
+        sqlfile = grad_mkfilename(grad_config_dir, "sqlserver");
         /* Open source file */
         if ((sqlfd = fopen(sqlfile, "r")) == (FILE *) NULL) {
                 grad_log(L_ERR, _("could not read sqlserver file %s"), sqlfile);
@@ -320,7 +320,7 @@ radiusd_sql_config()
         line_no = 0;
         cur_line = NULL;
         obstack_init(&parse_stack);
-        while (getline()) {
+        while (get_conf_line()) {
                 if (stmt_type == -1) {
                         grad_log(L_ERR,
                                  "%s:%d: %s",
@@ -1140,6 +1140,7 @@ rad_sql_retrieve_sessions(struct sql_connection *conn,
         for (i = 0; i < res->ntuples; i++) {
 		struct radutmp *up = grad_emalloc(sizeof(*up));
 		GRAD_STRING_COPY(up->login, res->tuple[i][0]);
+		GRAD_STRING_COPY(up->orig_login, res->tuple[i][0]);
 		up->nas_address = htonl(grad_ip_strtoip(res->tuple[i][1]));
 		up->nas_port = strtoul(res->tuple[i][2], NULL, 0);
 		GRAD_STRING_COPY(up->session_id, res->tuple[i][3]);
@@ -1216,7 +1217,7 @@ rad_sql_mlc_close(struct radutmp *up)
 	/* Create a temporary request */
 	req = grad_request_alloc();
 	grad_avl_add_pair(&req->avlist,
-			  grad_avp_create_string(DA_USER_NAME, up->login));
+			  grad_avp_create_string(DA_USER_NAME, up->orig_login));
 	grad_avl_add_pair(&req->avlist,
 			  grad_avp_create_integer(DA_NAS_PORT_ID,
 						  up->nas_port));
