@@ -106,7 +106,7 @@ rad_postgres_query(struct sql_connection *conn, char *query, int *return_count)
                 grad_log(L_ERR,
                          "PQexec: %s",
                          PQerrorMessage((PGconn*)conn->data));
-		if (PQstatus(conn) == CONNECTION_BAD)
+		if (PQstatus((PGconn*)conn->data) == CONNECTION_BAD)
 			rad_postgres_disconnect(conn, 0);
                 return -1;
         }
@@ -151,7 +151,7 @@ rad_postgres_query(struct sql_connection *conn, char *query, int *return_count)
                          _("PQexec returned %s"),
                          PQresStatus(stat));
                 if (stat == PGRES_FATAL_ERROR 
-                    && PQstatus(conn) == CONNECTION_BAD) 
+                    && PQstatus((PGconn*)conn->data) == CONNECTION_BAD) 
 			rad_postgres_disconnect(conn, 0);
                 rc = -1;
         }
@@ -176,7 +176,7 @@ rad_postgres_getpwd(struct sql_connection *conn, char *query)
                 grad_log(L_ERR,
                          "PQexec: %s",
                          PQerrorMessage((PGconn*)conn->data));
-		if (PQstatus(conn) == CONNECTION_BAD)
+		if (PQstatus((PGconn*)conn->data) == CONNECTION_BAD)
 			rad_postgres_disconnect(conn, 0);
                 return NULL;
         }
@@ -203,7 +203,7 @@ rad_postgres_getpwd(struct sql_connection *conn, char *query)
                          _("PQexec returned %s"),
                          PQresStatus(stat));
                 if (stat == PGRES_FATAL_ERROR
-                    && PQstatus(conn) == CONNECTION_BAD)
+                    && PQstatus((PGconn*)conn->data) == CONNECTION_BAD)
 			rad_postgres_disconnect(conn, 0);
         }
         PQclear(res);
@@ -254,7 +254,7 @@ rad_postgres_exec(struct sql_connection *conn, char *query)
                 grad_log(L_ERR,
                          "PQexec: %s",
                          PQerrorMessage((PGconn*)conn->data));
-		if (PQstatus(conn) == CONNECTION_BAD)
+		if (PQstatus((PGconn*)conn->data) == CONNECTION_BAD)
 			rad_postgres_disconnect(conn, 0);
                 return NULL;
         }
@@ -271,7 +271,7 @@ rad_postgres_exec(struct sql_connection *conn, char *query)
 			grad_log(L_ERR,
 			         _("PQexec returned %s"),
 			         PQresStatus(stat));
-			if (PQstatus(conn) == CONNECTION_BAD)
+			if (PQstatus((PGconn*)conn->data) == CONNECTION_BAD)
 				rad_postgres_disconnect(conn, 0);
 		}
                 return NULL;
